@@ -2,6 +2,7 @@
 #![recursion_limit = "1024"]
 #![feature(rustc_private)]
 #![allow(
+    clippy::elidable_lifetime_names,
     clippy::match_like_matches_macro,
     clippy::needless_lifetimes,
     clippy::single_element_loop,
@@ -12,8 +13,11 @@
 
 #[macro_use]
 mod macros;
+#[macro_use]
+mod snapshot;
 
 mod common;
+mod debug;
 
 use crate::common::visit::{AsIfPrinted, FlattenParens};
 use proc_macro2::{Delimiter, Group, Ident, Span, TokenStream};
@@ -24,8 +28,8 @@ use syn::visit_mut::VisitMut as _;
 use syn::{
     parse_quote, token, AngleBracketedGenericArguments, Arm, BinOp, Block, Expr, ExprArray,
     ExprAssign, ExprAsync, ExprAwait, ExprBinary, ExprBlock, ExprBreak, ExprCall, ExprCast,
-    ExprClosure, ExprConst, ExprContinue, ExprField, ExprForLoop, ExprIf, ExprIndex, ExprLit,
-    ExprLoop, ExprMacro, ExprMatch, ExprMethodCall, ExprPath, ExprRange, ExprRawAddr,
+    ExprClosure, ExprConst, ExprContinue, ExprField, ExprForLoop, ExprIf, ExprIndex, ExprLet,
+    ExprLit, ExprLoop, ExprMacro, ExprMatch, ExprMethodCall, ExprPath, ExprRange, ExprRawAddr,
     ExprReference, ExprReturn, ExprStruct, ExprTry, ExprTryBlock, ExprTuple, ExprUnary, ExprUnsafe,
     ExprWhile, ExprYield, GenericArgument, Label, Lifetime, Lit, LitInt, Macro, MacroDelimiter,
     Member, Pat, PatWild, Path, PathArguments, PathSegment, PointerMutability, QSelf, RangeLimits,
@@ -423,6 +427,53 @@ fn test_range_precedence() {
     syn::parse_str::<Expr>("x .. x ..").unwrap_err();
 }
 
+#[test]
+fn test_range_attrs() {
+    // Attributes are not allowed on range expressions starting with `..`
+    syn::parse_str::<Expr>("#[allow()] ..").unwrap_err();
+    syn::parse_str::<Expr>("#[allow()] .. hi").unwrap_err();
+
+    snapshot!("#[allow()] lo .. hi" as Expr, @r#"
+    Expr::Range {
+        start: Some(Expr::Path {
+            attrs: [
+                Attribute {
+                    style: AttrStyle::Outer,
+                    meta: Meta::List {
+                        path: Path {
+                            segments: [
+                                PathSegment {
+                                    ident: "allow",
+                                },
+                            ],
+                        },
+                        delimiter: MacroDelimiter::Paren,
+                        tokens: TokenStream(``),
+                    },
+                },
+            ],
+            path: Path {
+                segments: [
+                    PathSegment {
+                        ident: "lo",
+                    },
+                ],
+            },
+        }),
+        limits: RangeLimits::HalfOpen,
+        end: Some(Expr::Path {
+            path: Path {
+                segments: [
+                    PathSegment {
+                        ident: "hi",
+                    },
+                ],
+            },
+        }),
+    }
+    "#);
+}
+
 #[test]
 fn test_ranges_bailout() {
     syn::parse_str::<Expr>(".. ?").unwrap_err();
@@ -799,6 +850,7 @@ fn test_fixup() {
         quote! { (1 + 1).abs() },
         quote! { (lo..hi)[..] },
         quote! { (a..b)..(c..d) },
+        quote! { (x > ..) > x },
         quote! { (&mut fut).await },
         quote! { &mut (x as i32) },
         quote! { -(x as i32) },
@@ -828,6 +880,11 @@ fn test_fixup() {
         quote! { (1 < 2) == (3 < 4) },
         quote! { { (let _ = ()) } },
         quote! { (#[attr] thing).field },
+        quote! { #[attr] (1 + 1) },
+        quote! { #[attr] (x = 1) },
+        quote! { #[attr] (x += 1) },
+        quote! { #[attr] (1 as T) },
+        quote! { (return #[attr] (x + ..)).field },
         quote! { (self.f)() },
         quote! { (return)..=return },
         quote! { 1 + (return)..=1 + return },
@@ -836,7 +893,7 @@ fn test_fixup() {
         let original: Expr = syn::parse2(tokens).unwrap();
 
         let mut flat = original.clone();
-        FlattenParens.visit_expr_mut(&mut flat);
+        FlattenParens::combine_attrs().visit_expr_mut(&mut flat);
         let reconstructed: Expr = match syn::parse2(flat.to_token_stream()) {
             Ok(reconstructed) => reconstructed,
             Err(err) => panic!("failed to parse `{}`: {}", flat.to_token_stream(), err),
@@ -846,9 +903,9 @@ fn test_fixup() {
             original == reconstructed,
             "original: {}\n{:#?}\nreconstructed: {}\n{:#?}",
             original.to_token_stream(),
-            crate::macros::debug::Lite(&original),
+            crate::debug::Lite(&original),
             reconstructed.to_token_stream(),
-            crate::macros::debug::Lite(&reconstructed),
+            crate::debug::Lite(&reconstructed),
         );
     }
 }
@@ -1075,6 +1132,20 @@ fn test_permutations() -> ExitCode {
             }));
         });
 
+        // Expr::Let
+        iter(depth, &mut |expr| {
+            f(Expr::Let(ExprLet {
+                attrs: Vec::new(),
+                let_token: Token![let](span),
+                pat: Box::new(Pat::Wild(PatWild {
+                    attrs: Vec::new(),
+                    underscore_token: Token![_](span),
+                })),
+                eq_token: Token![=](span),
+                expr: Box::new(expr),
+            }));
+        });
+
         // Expr::Range
         f(Expr::Range(ExprRange {
             // `..`
@@ -1595,25 +1666,25 @@ fn test_permutations() -> ExitCode {
             fail!(
                 "failed to parse: {}\n{:#?}",
                 tokens,
-                crate::macros::debug::Lite(&original),
+                crate::debug::Lite(&original),
             );
         };
         AsIfPrinted.visit_expr_mut(&mut original);
-        FlattenParens.visit_expr_mut(&mut parsed);
+        FlattenParens::combine_attrs().visit_expr_mut(&mut parsed);
         if original != parsed {
             fail!(
                 "before: {}\n{:#?}\nafter: {}\n{:#?}",
                 tokens,
-                crate::macros::debug::Lite(&original),
+                crate::debug::Lite(&original),
                 parsed.to_token_stream(),
-                crate::macros::debug::Lite(&parsed),
+                crate::debug::Lite(&parsed),
             );
         }
         let mut tokens_no_paren = tokens.clone();
         FlattenParens::visit_token_stream_mut(&mut tokens_no_paren);
         if tokens.to_string() != tokens_no_paren.to_string() {
             if let Ok(mut parsed2) = syn::parse2::<Expr>(tokens_no_paren) {
-                FlattenParens.visit_expr_mut(&mut parsed2);
+                FlattenParens::combine_attrs().visit_expr_mut(&mut parsed2);
                 if original == parsed2 {
                     fail!("redundant parens: {}", tokens);
                 }
